@@ -74,7 +74,7 @@ func pathUser(b *backend) *framework.Path {
 	}
 }
 
-func (b *backend) pathCredsRead(ctx context.Context, req *logical.Request, d *framework.FieldData) (*logical.Response, error) {
+func (b *backend) pathCredsRead(ctx context.Context, req *logical.Request, d *framework.FieldData) (response *logical.Response, rerr error) {
 	roleName := d.Get("name").(string)
 
 	// Read the policy
@@ -144,6 +144,21 @@ func (b *backend) pathCredsRead(ctx context.Context, req *logical.Request, d *fr
 		}
 	}
 
+	defer func() {
+		obsType := ObservationTypeAWSCredentialCreateSuccess
+		if rerr != nil {
+			obsType = ObservationTypeAWSCredentialCreateFail
+		}
+
+		b.TryRecordObservationWithRequest(ctx, req, obsType, map[string]interface{}{
+			"role_name":        roleName,
+			"credential_types": role.CredentialTypes,
+			"ttl":              ttl,
+			"max_ttl":          maxTTL,
+			"is_sts":           credentialType != iamUserCred,
+		})
+	}()
+
 	switch credentialType {
 	case iamUserCred:
 		return b.secretAccessKeysCreate(ctx, req.Storage, req.DisplayName, roleName, role)
