@@ -1,4 +1,4 @@
-// Copyright (c) HashiCorp, Inc.
+// Copyright IBM Corp. 2016, 2025
 // SPDX-License-Identifier: BUSL-1.1
 
 package vault
@@ -6,6 +6,7 @@ package vault
 import (
 	"context"
 	"fmt"
+	"os"
 	"reflect"
 	"slices"
 	"sort"
@@ -13,6 +14,7 @@ import (
 
 	"github.com/armon/go-radix"
 	"github.com/hashicorp/go-multierror"
+	"github.com/hashicorp/go-secure-stdlib/parseutil"
 	"github.com/hashicorp/go-secure-stdlib/strutil"
 	"github.com/hashicorp/vault/helper/identity"
 	"github.com/hashicorp/vault/helper/namespace"
@@ -40,8 +42,10 @@ type ACL struct {
 }
 
 type PolicyCheckOpts struct {
-	RootPrivsRequired bool
-	Unauth            bool
+	RootPrivsRequired          bool
+	Unauth                     bool
+	CheckSourcePath            bool
+	RecoverAlternateCapability *logical.Operation
 }
 
 type AuthResults struct {
@@ -524,26 +528,25 @@ CHECK:
 			return
 		}
 
-		if len(permissions.DeniedParameters) == 0 {
-			goto ALLOWED_PARAMETERS
-		}
+		useLegacyMatching := os.Getenv("VAULT_LEGACY_EXACT_MATCHING_ON_LIST") != ""
 
-		// Check if all parameters have been denied
-		if _, ok := permissions.DeniedParameters["*"]; ok {
-			return
-		}
+		if len(permissions.DeniedParameters) > 0 {
+			// Check if all parameters have been denied
+			if _, ok := permissions.DeniedParameters["*"]; ok {
+				return
+			}
 
-		for parameter, value := range req.Data {
-			// Check if parameter has been explicitly denied
-			if valueSlice, ok := permissions.DeniedParameters[strings.ToLower(parameter)]; ok {
-				// If the value exists in denied values slice, deny
-				if valueInParameterList(value, valueSlice) {
-					return
+			for parameter, value := range req.Data {
+				// Check if parameter has been explicitly denied
+				if valueSlice, ok := permissions.DeniedParameters[strings.ToLower(parameter)]; ok {
+					// If the value exists in denied values slice, deny
+					if valueInDeniedParameterList(value, valueSlice, useLegacyMatching) {
+						return
+					}
 				}
 			}
 		}
 
-	ALLOWED_PARAMETERS:
 		// If we don't have any allowed parameters set, allow
 		if len(permissions.AllowedParameters) == 0 {
 			ret.Allowed = true
@@ -563,9 +566,9 @@ CHECK:
 				return
 			}
 
-			// If the value doesn't exists in the allowed values slice,
+			// If the value doesn't exist in the allowed values slice,
 			// deny
-			if ok && !valueInParameterList(value, valueSlice) {
+			if ok && !valueInAllowedParameterList(value, valueSlice, useLegacyMatching) {
 				return
 			}
 		}
@@ -777,7 +780,7 @@ func (c *Core) recordPolicyEvaluationObservation(ctx context.Context, te *logica
 	}
 }
 
-func (c *Core) performPolicyChecks(ctx context.Context, acl *ACL, te *logical.TokenEntry, req *logical.Request, inEntity *identity.Entity, opts *PolicyCheckOpts) *AuthResults {
+func (c *Core) performPolicyChecksSinglePath(ctx context.Context, acl *ACL, te *logical.TokenEntry, req *logical.Request, inEntity *identity.Entity, opts *PolicyCheckOpts) *AuthResults {
 	ret := new(AuthResults)
 
 	// First, perform normal ACL checks if requested. The only time no ACL
@@ -810,16 +813,89 @@ func (c *Core) performPolicyChecks(ctx context.Context, acl *ACL, te *logical.To
 	return ret
 }
 
-func valueInParameterList(v interface{}, list []interface{}) bool {
+func valueInAllowedParameterList(v interface{}, list []interface{}, useLegacyMatching bool) bool {
 	// Empty list is equivalent to the item always existing in the list
 	if len(list) == 0 {
 		return true
 	}
 
-	return valueInSlice(v, list)
+	if valueInParameterList(v, list) {
+		return true
+	}
+
+	if useLegacyMatching {
+		// prevent execution of the new behaviour if we're in legacy mode
+		return false
+	}
+
+	if vSlice, ok := v.([]interface{}); ok {
+		// when not running in legacy mode, we run a relaxed check for slices that verifies if all
+		// elements in the slice exist in the allowed list, as opposed to checking if the allowed
+		// list contains a single element that matches the entire slice (but this whole-slice match
+		// is still supported)
+		for _, v := range vSlice {
+			if !valueInParameterList(v, list) {
+				return false
+			}
+		}
+
+		return true
+	} else if vString, ok := v.(string); ok {
+		// At this point we don't know if the field is of framework.TypeCommaStringSlice, but we assume it is
+		// because failing to match a value because of it being in a comma-separated string is way more likely
+		// and worse than accidentally matching a substring of a string value.
+		if vSlice, err := parseutil.ParseCommaStringSlice(vString); err == nil {
+			for _, v := range vSlice {
+				if !valueInParameterList(v, list) {
+					return false
+				}
+			}
+			return true
+		}
+	}
+
+	return false
 }
 
-func valueInSlice(v interface{}, list []interface{}) bool {
+func valueInDeniedParameterList(v interface{}, list []interface{}, useLegacyMatching bool) bool {
+	// Empty list is equivalent to the item always existing in the list
+	if len(list) == 0 {
+		return true
+	}
+
+	if valueInParameterList(v, list) {
+		return true
+	}
+
+	if useLegacyMatching {
+		// prevent execution of the new behaviour if we're in legacy mode
+		return false
+	}
+
+	// The new behaviour is that if any value in the slice is in the denied list, we deny.
+	if vSlice, ok := v.([]interface{}); ok {
+		for _, v := range vSlice {
+			if valueInParameterList(v, list) {
+				return true
+			}
+		}
+	} else if vString, ok := v.(string); ok {
+		// At this point we don't know if the field is of framework.TypeCommaStringSlice, but we assume it is
+		// because failing to match a value because of it being in a comma-separated string is way more likely
+		// and worse than accidentally matching a substring of a string value.
+		if vSlice, err := parseutil.ParseCommaStringSlice(vString); err == nil {
+			for _, v := range vSlice {
+				if valueInParameterList(v, list) {
+					return true
+				}
+			}
+		}
+	}
+
+	return false
+}
+
+func valueInParameterList(v interface{}, list []interface{}) bool {
 	for _, el := range list {
 		if el == nil || v == nil {
 			// It doesn't seem possible to set up a nil entry in the list, but it is possible
@@ -828,11 +904,8 @@ func valueInSlice(v interface{}, list []interface{}) bool {
 			if el == v {
 				return true
 			}
-		} else if reflect.TypeOf(el).String() == "string" && reflect.TypeOf(v).String() == "string" {
-			item := el.(string)
-			val := v.(string)
-
-			if strutil.GlobbedStringsMatch(item, val) {
+		} else if elStr, ok := el.(string); ok {
+			if vStr, ok := v.(string); ok && strutil.GlobbedStringsMatch(elStr, vStr) {
 				return true
 			}
 		} else if reflect.DeepEqual(el, v) {
